@@ -12,8 +12,7 @@
  *                                                                             *
  *******************************************************************************
  */
-void TimeAdjust(int time_used, PLAYER player)
-{
+void TimeAdjust(int time_used, int side) {
 /*
  ************************************************************
  *                                                          *
@@ -23,27 +22,19 @@ void TimeAdjust(int time_used, PLAYER player)
  *                                                          *
  ************************************************************
  */
-  if (player == crafty) {
-    tc_moves_remaining--;
-    tc_time_remaining -=
-        (tc_time_remaining > time_used) ? time_used : tc_time_remaining;
-    if (!tc_moves_remaining) {
-      if (tc_sudden_death == 2)
-        tc_sudden_death = 1;
-      tc_moves_remaining += tc_secondary_moves;
-      tc_time_remaining += tc_secondary_time;
-      tc_time_remaining_opponent += tc_secondary_time;
-      Print(4095, "time control reached\n");
-    }
-    if (tc_increment)
-      tc_time_remaining += tc_increment;
-  } else {
-    tc_time_remaining_opponent -=
-        (tc_time_remaining_opponent >
-        time_used) ? time_used : tc_time_remaining_opponent;
-    if (tc_increment)
-      tc_time_remaining_opponent += tc_increment;
+  tc_moves_remaining[side]--;
+  tc_time_remaining[side] -=
+      (tc_time_remaining[side] >
+      time_used) ? time_used : tc_time_remaining[side];
+  if (!tc_moves_remaining[side]) {
+    if (tc_sudden_death == 2)
+      tc_sudden_death = 1;
+    tc_moves_remaining[side] += tc_secondary_moves;
+    tc_time_remaining[side] += tc_secondary_time;
+    Print(4095, "time control reached (%s)\n", (side) ? "white" : "black");
   }
+  if (tc_increment)
+    tc_time_remaining[side] += tc_increment;
 }
 
 /* last modified 01/17/09 */
@@ -59,21 +50,55 @@ void TimeAdjust(int time_used, PLAYER player)
  *   extended based on how far the root value has dropped in an effort to      *
  *   avoid whatever is being lost.                                             *
  *                                                                             *
+ *   We use one additional trick here to avoid stopping the search just before *
+ *   we change to a better move.  Once we reach the time limit, we set do not  *
+ *   immediately stop the search, rather, we let it continue to any/all root   *
+ *   moves that are "in progress" complete.  This is important, particularly   *
+ *   on a parallel search that splits at the root like Crafty does.  A new     *
+ *   best move will take quite a bit of time to search, and with just one CPU  *
+ *   working on it, we might not have time to complete it before the search    *
+ *   time limit is reached.  Once we reach this point, we set a flag that      *
+ *   says "do not start searching a new root move, but do not abort any active *
+ *   search.  As processors finish their root moves, they will begin to help   *
+ *   those with incomplete root move searches to make sure that none of them   *
+ *   will become a new best move.  This way, once we start searching any root  *
+ *   move, we will not give up on it until the search has completed and the    *
+ *   move was proved worse than the best root move so far, or until we         *
+ *   discover that this root move is actually better.                          *
+ *                                                                             *
+ *   This is implemented by having Search() call TimeCheck() passing it a      *
+ *   value of zero (0) for the parameter abort.  TimeCheck() will only end the *
+ *   search if we have exceeded the max time limit or we have not gotten a     *
+ *   best score at the root on the current iteration and we have reached the   *
+ *   normal time limit.  Otherwise TimeCheck() will return the "abort" value   *
+ *   which is always zero when called from Search().  We also call TimeCheck() *
+ *   from NextRootMove() and it passes TimeCheck() a value of 1 for the abort  *
+ *   flag.  Once anyone posts the "abort" flag, NextRootMove() will return a   *
+ *   indication saying "no more root moves to search", which will eventually   *
+ *   end the search when all current root moves are completed and              *
+ *   NextRootMove() is called to obtain another root move to search.           *
+ *                                                                             *
+ *   The global variable "time_abort" is initially set to zero, and so long    *
+ *   as it is zero, NextRootMove() will continue to return moves to search     *
+ *   until the iteration ends.  Whenever the time_abort flag becomes non-zero, *
+ *   NextRootMove() refuses to search any new moves, but the current searches  *
+ *   are allowed to continue until they all complete or the hard time limit    *
+ *   (absolute_time_limit) is reached where the search is terminated           *
+ *   immediately to avoid overstepping the time control limits.                *
+ *                                                                             *
  *******************************************************************************
  */
-int TimeCheck(TREE * RESTRICT tree, int abort)
-{
+int TimeCheck(TREE * RESTRICT tree, int abort) {
   int time_used;
-  int value, last_value;
   int i, ndone;
 
 /*
  ************************************************************
  *                                                          *
- *   First, check to see if we are searching the first move *
- *   at this depth.  If so, and we run out of time, we can  *
- *   abort the search rather than waiting to complete this  *
- *   ply=1 move to see if it's better.                      *
+ *   First, check to see if we are are supposed to stop     *
+ *   after a fixed number of nodes.  If not, check to see   *
+ *   if there is just one legal move which lets us stop     *
+ *   quickly since there is no choice to be made.           *
  *                                                          *
  ************************************************************
  */
@@ -82,6 +107,16 @@ int TimeCheck(TREE * RESTRICT tree, int abort)
   if (n_root_moves == 1 && !booking && !annotate_mode && !pondering &&
       iteration_depth > 4)
     return (1);
+/*
+ ************************************************************
+ *                                                          *
+ *   Now, check to see if we are searching the first move   *
+ *   at this depth.  If so, and we run out of time, we can  *
+ *   abort the search rather than waiting to complete this  *
+ *   ply=1 move to see if it's better.                      *
+ *                                                          *
+ ************************************************************
+ */
   ndone = 0;
   for (i = 0; i < n_root_moves; i++)
     if (root_moves[i].status & 256)
@@ -93,9 +128,9 @@ int TimeCheck(TREE * RESTRICT tree, int abort)
 /*
  ************************************************************
  *                                                          *
- *   Now, check to see if we need to "burp" the time to     *
- *   let the operator know the search is progressing and    *
- *   how much time has been used so far.                    *
+ *   Check to see if we need to "burp" the time to let the  *
+ *   operator know the search is progressing and how much   *
+ *   time has been used so far.                             *
  *                                                          *
  ************************************************************
  */
@@ -119,12 +154,32 @@ int TimeCheck(TREE * RESTRICT tree, int abort)
     fflush(stdout);
     Unlock(lock_io);
   }
+/*
+ ************************************************************
+ *                                                          *
+ *   If we are pondering or in analyze mode, we do not      *
+ *   terminate on time since there is no time limit placed  *
+ *   on these searches.  If we have reached the absolute    *
+ *   time limit, we stop the search instantly.              *
+ *                                                          *
+ *   If we are under the time limit already set, we do not  *
+ *   terminate the search.  If the operator set a specific  *
+ *   search time limit, we stop when we hit that regardless *
+ *   of the score.                                          *
+ *                                                          *
+ *   The only other case is an "easy move" which is a move  *
+ *   that looks significantly better than the rest of the   *
+ *   root moves when they are initially ordered, and this   *
+ *   move does not fail low during subsequent searches.     *
+ *                                                          *
+ ************************************************************
+ */
   if (pondering || analyze_mode)
     return (0);
   if (time_used > absolute_time_limit)
     return (1);
   if (easy_move && !search_time_limit) {
-    if (time_limit > 100 && time_used >= time_limit / 3)
+    if (time_used >= (36 * time_limit) / 100)   //TLR if (time_limit > 100 && time_used >= time_limit / 3)
       return (1);
   }
   if (time_used < time_limit)
@@ -149,51 +204,41 @@ int TimeCheck(TREE * RESTRICT tree, int abort)
  ************************************************************
  *                                                          *
  *   We have a score at the root of the tree, if the        *
- *   evaluation is not significantly worse than the last    *
- *   evaluation (from the previous iteration...) then we    *
- *   safely stop the search.  Note also that if the current *
- *   evaluation is quite a bit worse, but we are still way  *
- *   ahead, we can still avoid using extra time.            *
+ *   evaluation is not worse than the last evaluation       *
+ *   (from the previous iteration...) then we safely stop   *
+ *   the search.                                            *
  *                                                          *
  ************************************************************
  */
-  value = root_value;
-  last_value = last_root_value;
-  if ((value >= last_value - 24 && !(root_moves[0].status & 7)) || (value > 350
-          && value >= last_value - 50)) {
-    if (time_used > time_limit * 2)
-      return (1);
-    else
-      return (abort);
-  }
+  if ((root_value >= last_root_value && !(root_moves[0].status & 7)))
+    return (abort);
 /*
  ************************************************************
  *                                                          *
  *   We are in trouble at the root.  Depending on how much  *
  *   the score has dropped, increase the search time limit  *
- *   to try and correct the problem.  For a positional drop *
- *   we can double the search time (this is for a serious   *
- *   drop, of course).                                      *
+ *   to try and correct the problem.  Time can be increased *
+ *   by a factor of 2, 4, or 8 based on how low the score   *
+ *   actually dropped.                                      *
  *                                                          *
  ************************************************************
  */
-  if (time_used < time_limit * 2.5 && time_used + 500 < tc_time_remaining)
+  if ((root_value >= last_root_value - 25 && !(root_moves[0].status & 7))) {
+    if (time_used > time_limit * 2 ||
+        time_used + 500 > tc_time_remaining[root_wtm])
+      return (abort);
     return (0);
-  if ((value >= last_value - 67 && !(root_moves[0].status & 7)) || value > 550)
-    return (abort);
-/*
- ************************************************************
- *                                                          *
- *   We are in really serious trouble at the root, losing   *
- *   material.  Increase the time limit to 6X the original  *
- *   target, as losing material is tantamount to losing the *
- *   game anyway.                                           *
- *                                                          *
- ************************************************************
- */
-  if (time_used < time_limit * 7 && time_used + 500 < tc_time_remaining)
+  }
+  if ((root_value >= last_root_value - 50 && !(root_moves[0].status & 7))) {
+    if (time_used > time_limit * 4 ||
+        time_used + 400 > tc_time_remaining[root_wtm])
+      return (abort);
     return (0);
-  return (1);
+  }
+  if (time_used > time_limit * 8 ||
+      time_used + 300 > tc_time_remaining[root_wtm])
+    return (1);
+  return (0);
 }
 
 /* last modified 01/17/09 */
@@ -208,13 +253,25 @@ int TimeCheck(TREE * RESTRICT tree, int abort)
  *                                                                             *
  *******************************************************************************
  */
-void TimeSet(int search_type)
-{
+static int moves_left[64] = {
+  18, 18, 18, 18, 18, 18, 18, 18,
+  19, 19, 19, 19, 20, 20, 20, 20,
+  21, 21, 21, 21, 22, 22, 22, 22,
+  23, 23, 23, 23, 24, 24, 24, 24,
+  25, 25, 25, 25, 26, 26, 26, 26,
+  27, 27, 27, 27, 28, 28, 28, 28,
+  29, 29, 29, 29, 30, 30, 30, 30,
+  31, 31, 31, 31, 32, 32, 32, 32
+};
+void TimeSet(TREE * RESTRICT tree, int search_type) {
   static const float behind[6] = { 32.0, 16.0, 8.0, 4.0, 2.0, 1.5 };
   static const int reduce[6] = { 96, 48, 24, 12, 6, 3 };
   int i, mult = 0, extra = 0;
   int surplus, average;
   int simple_average;
+  int phase =
+      moves_left[Min(63, TotalPieces(white, occupied) + TotalPieces(black,
+              occupied))] - 4;
 
   surplus = 0;
   average = 0;
@@ -244,21 +301,35 @@ void TimeSet(int search_type)
   if (tc_sudden_death == 1) {
     if (tc_increment) {
       time_limit =
-          (tc_time_remaining -
-          tc_operator_time * tc_moves_remaining) / (ponder ? 18 : 26) +
-          tc_increment;
-      if (tc_time_remaining < 500 + tc_increment) {
+          (tc_time_remaining[root_wtm] -
+          tc_operator_time * tc_moves_remaining[root_wtm]) /
+          (ponder ? phase : phase + 6) + tc_increment;
+      if (tc_time_remaining[root_wtm] < 500 + tc_increment) {
         time_limit = tc_increment;
-        if (tc_time_remaining < 250 + tc_increment)
+        if (tc_time_remaining[root_wtm] < 250 + tc_increment)
           time_limit /= 2;
       }
-      absolute_time_limit = tc_time_remaining / 2;
+      absolute_time_limit = tc_time_remaining[root_wtm] / 2 + tc_increment;
       if (absolute_time_limit < time_limit ||
-          tc_time_remaining - time_limit < 100)
+          tc_time_remaining[root_wtm] - time_limit < 100)
         absolute_time_limit = time_limit;
+     // Safety.
+      if (tc_time_remaining[root_wtm] - time_limit < 50) {
+        time_limit = tc_time_remaining[root_wtm] - 50;
+        if (time_limit < 5)
+          time_limit = 5;
+      }
+      if (tc_time_remaining[root_wtm] - absolute_time_limit < 25) {
+        absolute_time_limit = tc_time_remaining[root_wtm] - 25;
+        if (absolute_time_limit < 5)
+          absolute_time_limit = 5;
+      }
+
     } else {
-      time_limit = tc_time_remaining / (ponder ? 27 : 33);
-      absolute_time_limit = Min(time_limit * 6, tc_time_remaining / 2);
+      time_limit =
+          tc_time_remaining[root_wtm] / (ponder ? phase + 6 : phase + 12);
+      absolute_time_limit =
+          Min(time_limit * 5, tc_time_remaining[root_wtm] / 2);
     }
   }
 /*
@@ -278,18 +349,22 @@ void TimeSet(int search_type)
   else {
     if (move_number <= tc_moves)
       simple_average =
-          (tc_time - (tc_operator_time * tc_moves_remaining)) / tc_moves;
+          (tc_time -
+          (tc_operator_time * tc_moves_remaining[root_wtm])) / tc_moves;
     else
       simple_average =
           (tc_secondary_time -
-          (tc_operator_time * tc_moves_remaining)) / tc_secondary_moves;
+          (tc_operator_time * tc_moves_remaining[root_wtm])) /
+          tc_secondary_moves;
     surplus =
-        Max(tc_time_remaining - (tc_operator_time * tc_moves_remaining) -
-        simple_average * tc_moves_remaining, 0);
+        Max(tc_time_remaining[root_wtm] -
+        (tc_operator_time * tc_moves_remaining[root_wtm]) -
+        simple_average * tc_moves_remaining[root_wtm], 0);
     average =
-        (tc_time_remaining - (tc_operator_time * tc_moves_remaining) +
-        tc_moves_remaining * tc_increment)
-        / tc_moves_remaining;
+        (tc_time_remaining[root_wtm] -
+        (tc_operator_time * tc_moves_remaining[root_wtm]) +
+        tc_moves_remaining[root_wtm] * tc_increment)
+        / tc_moves_remaining[root_wtm];
     if (surplus < tc_safety_margin)
       time_limit = (average < simple_average) ? average : simple_average;
     else
@@ -303,12 +378,12 @@ void TimeSet(int search_type)
   if (time_limit <= 0)
     time_limit = 5;
   absolute_time_limit =
-      time_limit + surplus / 2 + ((tc_time_remaining -
-          tc_operator_time * tc_moves_remaining) / 4);
-  if (absolute_time_limit > 7 * time_limit)
-    absolute_time_limit = 7 * time_limit;
-  if (absolute_time_limit > tc_time_remaining / 2)
-    absolute_time_limit = tc_time_remaining / 2;
+      time_limit + surplus / 2 + ((tc_time_remaining[root_wtm] -
+          tc_operator_time * tc_moves_remaining[root_wtm]) / 4);
+  if (absolute_time_limit > 6 * time_limit)
+    absolute_time_limit = 6 * time_limit;
+  if (absolute_time_limit > tc_time_remaining[root_wtm] / 2)
+    absolute_time_limit = tc_time_remaining[root_wtm] / 2;
 /*
  ************************************************************
  *                                                          *
@@ -323,10 +398,9 @@ void TimeSet(int search_type)
  */
   if (usage_level)
     time_limit *= 1.0 + usage_level / 100.0;
-  if (!ponder)
-    time_limit = 3 * time_limit / 4;
   if (first_nonbook_factor && moves_out_of_book < first_nonbook_span) {
-    mult = (first_nonbook_span - moves_out_of_book + 1) * first_nonbook_factor;
+    mult =
+        (first_nonbook_span - moves_out_of_book + 1) * first_nonbook_factor;
     extra = time_limit * mult / first_nonbook_span / 100;
     time_limit += extra;
   }
@@ -334,9 +408,9 @@ void TimeSet(int search_type)
  ************************************************************
  *                                                          *
  *  This code is used to handle the case where someone is   *
- *  trying to "blitz" crafty by reaching a position where   *
+ *  trying to "blitz" Crafty by reaching a position where   *
  *  things are locked up, and then just shuffling pieces    *
- *  back and forth.  When crafty reaches the point where it *
+ *  back and forth.  When Crafty reaches the point where it *
  *  has less than 3/4 of the time the opponent has, it      *
  *  starts decreasing the target time.  At 1/2, it          *
  *  decreases it further.                                   *
@@ -345,20 +419,21 @@ void TimeSet(int search_type)
  */
   if (mode != tournament_mode && !computer_opponent) {
     for (i = 0; i < 6; i++) {
-      if ((float) tc_time_remaining * behind[i] <
-          (float) tc_time_remaining_opponent) {
+      if ((float) tc_time_remaining[root_wtm] * behind[i] <
+          (float) tc_time_remaining[Flip(root_wtm)]) {
         time_limit = time_limit / reduce[i];
-        Print(128, "crafty is behind %4.1f on time, reducing by 1/%d.\n",
+        Print(128, "Crafty is behind %4.1f on time, reducing by 1/%d.\n",
             behind[i], reduce[i]);
         break;
       }
     }
-    if (tc_increment == 0 && tc_time_remaining_opponent > tc_time_remaining) {
-      if (tc_time_remaining < 3000)
+    if (tc_increment == 0 &&
+        tc_time_remaining[Flip(root_wtm)] > tc_time_remaining[root_wtm]) {
+      if (tc_time_remaining[root_wtm] < 3000)
         time_limit /= 2;
-      if (tc_time_remaining < 2000)
+      if (tc_time_remaining[root_wtm] < 2000)
         time_limit /= 2;
-      if (tc_time_remaining < 1000)
+      if (tc_time_remaining[root_wtm] < 1000)
         time_limit = 1;
     }
   }
